/****************************************************************************
 * <Novell-copyright>
 * Copyright (c) 2001 Novell, Inc. All Rights Reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, contact Novell, Inc.
 * 
 * To contact Novell about this file by physical or electronic mail, you 
 * may find current contact information at www.novell.com.
 * </Novell-copyright>
 ****************************************************************************/

#include <config.h>
#include <xpl.h>
#include "imapd.h"

static StoreCommandFlag StoreCommandFlags[] = {
    { "\\Deleted", sizeof("\\Deleted") - 1, MSG_STATE_DELETED },
    { "\\Seen", sizeof("\\Seen") - 1, MSG_STATE_READ },
    { "\\Recent", sizeof("\\Recent") - 1, MSG_STATE_RECENT },
    { "\\Draft", sizeof("\\Draft") - 1, MSG_STATE_DRAFT },
    { "\\Answered", sizeof("\\Answered") - 1, MSG_STATE_ANSWERED },
    { "\\Flagged", sizeof("\\Flagged") - 1, MSG_STATE_PRIOHIGH },
    { NULL, 0, 0 }
};

static StoreCommandType StoreCommandTypes[] = {
    { "FLAGS.SILENT", "SFLG", TRUE },
    { "+FLAGS.SILENT", "AFLG", TRUE },
    { "-FLAGS.SILENT", "DFLG", TRUE },
    { "FLAGS", "SFLG", FALSE },
    { "+FLAGS", "AFLG", FALSE },
    { "-FLAGS", "DFLG", FALSE },
    { NULL, NULL, FALSE }
};

unsigned long
ParseStoreFlags(unsigned char *command, unsigned long *returnFlags)
{
    unsigned char *ptr;
    unsigned long flags;
    long ccode;

    flags = 0;
    ptr = command;

    do {
        if (*ptr) {
            ccode = HulaKeywordFind(Imap.command.store.flagIndex, ptr);
            if (ccode != -1) {
                flags |= StoreCommandFlags[ccode].value;
                ptr += StoreCommandFlags[ccode].stringLen;
                if (*ptr == ' ') {
                    ptr++;
                }
                continue;
            }
            return(STATUS_INVALID_ARGUMENT);
        }
        break;
    } while (TRUE);

    *returnFlags = flags;
    return(STATUS_CONTINUE);
}

BOOL
CommandStoreInit()
{
    HulaKeywordIndexCreateFromTable(Imap.command.store.typeIndex, StoreCommandTypes, .string, TRUE);
    if (Imap.command.store.typeIndex) {
        HulaKeywordIndexCreateFromTable(Imap.command.store.flagIndex, StoreCommandFlags, .string, TRUE);
        if (Imap.command.store.flagIndex) {
            return(TRUE);
        }
        HulaKeywordIndexFree(Imap.command.store.typeIndex);
    }
    return(FALSE);
}

void
CommandStoreCleanup()
{
    HulaKeywordIndexFree(Imap.command.store.typeIndex);
    HulaKeywordIndexFree(Imap.command.store.flagIndex);
}


__inline static long
DoStoreForMessageRange(ImapClient *client, char *nmapCommand, unsigned long flags, unsigned long rangeStart, unsigned long rangeEnd, BOOL silent, BOOL byUid)
{
    unsigned long messageId;
    long messageFlags;
    long ccode;
    char whiteSpace[2];

    messageId = rangeStart;

    do {
        /* Do the dirty work */

        if (NMAPSendCommandF(client->nmap, "%s %lu %lu\r\n", nmapCommand, client->IDList[messageId], flags) != -1) {
            ccode = NMAPReadResponseAndCount(client->nmap, &messageFlags);
            switch(ccode) {
                case 1000: {
                    if (!silent) {
                        ccode = ConnWriteF(client->conn, "* %lu FETCH (FLAGS (", messageId + 1);
                        whiteSpace[0] = '\0';
                        whiteSpace[1] = '\0';
                        if ((messageFlags & MSG_STATE_ANSWERED) && (ccode != -1)) {
                            ccode = ConnWrite(client->conn, "\\Answered", sizeof("\\Answered") - 1);
                            whiteSpace[0] = ' ';
                        } 
                        if ((messageFlags & MSG_STATE_DELETED) && (ccode != -1)) {
                            ccode = ConnWriteF(client->conn, "%s\\Deleted",whiteSpace);
                            whiteSpace[0] = ' ';
                        } 
                        if ((messageFlags & MSG_STATE_RECENT) && (ccode != -1)) {
                            ccode = ConnWriteF(client->conn, "%s\\Recent",whiteSpace);
                            whiteSpace[0] = ' ';
                        } 
                        if ((messageFlags & MSG_STATE_READ) && (ccode != -1)) {
                            ccode = ConnWriteF(client->conn, "%s\\Seen",whiteSpace);
                            whiteSpace[0] = ' ';
                        }
                        if ((messageFlags & MSG_STATE_DRAFT) && (ccode != -1)) {
                            ccode = ConnWriteF(client->conn, "%s\\Draft",whiteSpace);
                            whiteSpace[0] = ' ';
                        }
                        if ((messageFlags & MSG_STATE_PRIOHIGH) && (ccode != -1)) {
                            ccode = ConnWriteF(client->conn, "%s\\Flagged",whiteSpace);
                            whiteSpace[0] = ' ';
                        }
                        if (ccode != -1) {
                            if (byUid) {
                                ccode = ConnWriteF(client->conn, ") UID %lu)\r\n", client->UIDList[messageId]);
                            } else {
                                ccode = ConnWrite(client->conn, "))\r\n", sizeof("))\r\n") - 1);
                            }
                        }
                        if (ccode != -1) {
                            break;
                        }

                        return(STATUS_ABORT);
                    }
                    break;
                }

                case NMAP_FAILED_READONLY: {
                    if (silent) {
                        if (NMAPSendCommandF(client->nmap, "GFLG %lu\r\n", client->IDList[messageId]) != -1) {
                            ccode = NMAPReadResponseAndCount(client->nmap, &messageFlags);
                            if (ccode != 1000) {
                                ccode = ConnWriteF(client->conn, "* %lu FETCH (FLAGS (", messageId + 1);
                                whiteSpace[0] = '\0';
                                whiteSpace[1] = '\0';
                                if ((messageFlags & MSG_STATE_ANSWERED) && (ccode != -1)) {
                                    ccode = ConnWrite(client->conn, "\\Answered", sizeof("\\Answered") - 1);
                                    whiteSpace[0] = ' ';
                                } 
                                if ((messageFlags & MSG_STATE_DELETED) && (ccode != -1)) {
                                    ccode = ConnWriteF(client->conn, "%s\\Deleted",whiteSpace);
                                    whiteSpace[0] = ' ';
                                } 
                                if ((messageFlags & MSG_STATE_RECENT) && (ccode != -1)) {
                                    ccode = ConnWriteF(client->conn, "%s\\Recent",whiteSpace);
                                    whiteSpace[0] = ' ';
                                } 
                                if ((messageFlags & MSG_STATE_READ) && (ccode != -1)) {
                                    ccode = ConnWriteF(client->conn, "%s\\Seen",whiteSpace);
                                    whiteSpace[0] = ' ';
                                }
                                if ((messageFlags & MSG_STATE_DRAFT) && (ccode != -1)) {
                                    ccode = ConnWriteF(client->conn, "%s\\Draft",whiteSpace);
                                    whiteSpace[0] = ' ';
                                }
                                if ((messageFlags & MSG_STATE_PRIOHIGH) && (ccode != -1)) {
                                    ccode = ConnWriteF(client->conn, "%s\\Flagged",whiteSpace);
                                    whiteSpace[0] = ' ';
                                }
                                if (ccode != -1) {
                                    if (byUid) {
                                        ccode = ConnWriteF(client->conn, ") UID %lu)\r\n", client->UIDList[messageId]);
                                    } else {
                                        ccode = ConnWrite(client->conn, "))\r\n", sizeof("))\r\n") - 1);
                                    }
                                }
                                if (ccode != -1) {
                                    break;
                                }
                                return(STATUS_ABORT);
                            }
                            return(CheckForNMAPCommError(ccode));
                        }
                        return(STATUS_NMAP_COMM_ERROR);
                    }
                    break;
                }

                default: {
                    return(CheckForNMAPCommError(ccode));
                }
            }

            /* Should we terminate the while loop ? */
            if (messageId < rangeEnd) {
                messageId++;
                continue;
            }
            break;
        }
        return(STATUS_NMAP_COMM_ERROR);
    } while (TRUE);

    return(STATUS_CONTINUE);
}

__inline static long
HandleStore(ImapClient *client, BOOL ByUID)
{
    unsigned char *messageSet;
    char *nextRange;
    unsigned long rangeStart;
    unsigned long rangeEnd;
    long ccode;
    unsigned char *ptr;
    unsigned char *Type;
    unsigned char *Value;
    unsigned char *Command;
    unsigned char *NMAPCommand;
    BOOL Silent = FALSE;
 
    /* Message attributes/values*/
    unsigned long Flags;

    if (ByUID) {
        Command = "UID STORE";
    } else {
        Command = "STORE";
    }

    ptr = client->Command + 5;

    /* Grab the range */
    ccode = GrabArgument(client, &ptr, &messageSet);
    if (ccode == STATUS_CONTINUE) {
        /* Grab the way the flags are to be set */
        ccode = GrabArgument(client, &ptr, &Type);
        if (ccode == STATUS_CONTINUE) {
            ccode = HulaKeywordFind(Imap.command.store.typeIndex, Type);
            MemFree(Type);
            if (ccode != -1) {
                NMAPCommand = StoreCommandTypes[ccode].nmapCommand;
                Silent = StoreCommandTypes[ccode].silent;
                /* Grab the flag values */
                ccode = GrabArgument(client, &ptr, &Value);
                if (ccode == STATUS_CONTINUE) {
                    ccode = ParseStoreFlags(Value, &Flags);
                    MemFree(Value);
                    if (ccode == STATUS_CONTINUE) {
                        nextRange = messageSet;
                        do {
                            ccode = GetMessageRange(client, &(nextRange), &rangeStart, &rangeEnd, ByUID);
                            if (ccode == STATUS_CONTINUE) {
                                ccode = DoStoreForMessageRange(client, NMAPCommand, Flags, rangeStart, rangeEnd, Silent, ByUID);
                                if (ccode == STATUS_CONTINUE) {
                                    continue;
                                }
                            }
                            if (ccode == STATUS_UID_NOT_FOUND) {
                                continue;
                            }
                            MemFree(messageSet);
                            return(SendError(client->conn, client->Ident, Command, ccode));
                        } while (nextRange);
                        MemFree(messageSet);
                        return(SendOk(client, Command));
                    }
                }
                MemFree(messageSet);
                return(SendError(client->conn, client->Ident, Command, ccode));
            }
            MemFree(messageSet);
            return(SendError(client->conn, client->Ident, Command, STATUS_INVALID_ARGUMENT));
        }
        MemFree(messageSet);
    }

    return(SendError(client->conn, client->Ident, Command, ccode));
}

int
ImapCommandStore(void *param)
{
    long ccode;

    ImapClient *client = (ImapClient *)param;

    if (client->state == STATE_SELECTED) {
        /* rfc 2180 discourages purge notifications during the store command */
        if ((ccode = NewMessageNotify(client)) == STATUS_CONTINUE) {
            return(HandleStore(client, FALSE));
        }

        return(SendError(client->conn, client->Ident, "STORE", ccode));
    }

    return(SendError(client->conn, client->Ident, "STORE", STATUS_INVALID_STATE));
}

int
ImapCommandUidStore(void *param)
{
    long ccode;
    ImapClient *client = (ImapClient *)param;
    unsigned char *ptr;

    if (client->state == STATE_SELECTED) {
        /* rfc 2180 discourages purge notifications during the store command */    
        if ((ccode = NewMessageNotify(client)) == STATUS_CONTINUE) {
            ptr = client->Command + 4;
            memmove(client->Command, ptr, strlen(ptr) + 1);
            return(HandleStore(client, TRUE));
        }

        return(SendError(client->conn, client->Ident, "UID STORE", ccode));
    }

    return(SendError(client->conn, client->Ident, "UID STORE", STATUS_INVALID_STATE));
}
