/****************************************************************************
 * <Novell-copyright>
 * Copyright (c) 2001-2005 Novell, Inc. All Rights Reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, contact Novell, Inc.
 * 
 * To contact Novell about this file by physical or electronic mail, you 
 * may find current contact information at www.novell.com.
 * </Novell-copyright>
 ****************************************************************************/

#include <config.h>
#include <logger.h>

#include "nmapdp.h"
#include "nmlib.h"

void 
FreeClientData(NMAPClient *client)
{
    unsigned int i;
    struct stat sb;
    sqlite3_stmt **stmt;

    if (!(client->states & NMAP_CLIENT_EXITING)) {
        client->states |= NMAP_CLIENT_EXITING;

        /* Old cleanup code */

        /* Check if new mail arrived or if the status of any message changed; 
           prevents loss of new messages since we call StoreMaildrop below */
        if (client->states & NMAP_CLIENT_MBOX) {
            if (!(client->share.flags & NMAP_SHARE_MAILBOX)) {
                sprintf(client->path, "%s/%s/%s.box", client->store, client->user, client->mailbox.name);
                if ((stat(client->path, &sb) == 0) && (client->mailbox.size != (unsigned long)sb.st_size)) {
                    ParseMaildrop(client);                                                                 
                }

                if (client->mailbox.fh) {
                    fclose(client->mailbox.fh);
                    client->mailbox.fh = NULL;
                }

                StoreMaildrop(client);
            } else {
                StoreSharedMaildrop(client);

                ConnClose(client->share.mailbox.conn, 0);
                ConnFree(client->share.mailbox.conn);
                client->share.mailbox.conn = NULL;
            }
        }

        if (client->mailbox.lock) {
            ReadNLockRelease(client->mailbox.lock);
            client->mailbox.lock = NULL;
        }

        if (client->mailbox.message.info) {
            MemFree(client->mailbox.message.info);
            client->mailbox.message.info = NULL;
        }

        if (client->calendar.name[0] && !(client->share.flags & NMAP_SHARE_CALENDAR)) {
            if (client->calendar.fh) {
                fclose(client->calendar.fh);
                client->calendar.fh = NULL;
            }

            StoreCalendar(client);
        } else if (client->share.flags & NMAP_SHARE_CALENDAR) {
            StoreSharedCalendar(client);

            ConnClose(client->share.calendar.conn, 0);
            ConnFree(client->share.calendar.conn);
            client->share.calendar.conn = NULL;
        }

        if (client->calendar.lock) {
            ReadNLockRelease(client->calendar.lock);
            client->calendar.lock = NULL;
        }

        if (client->calendar.entries.info) {
            MemFree(client->calendar.entries.info);
            client->calendar.entries.info = NULL;
        }

        stmt = &(client->dstore.sql.begin);
        for (i = 0; i < (sizeof(NmapClientDStoreStatements) / sizeof(sqlite3_stmt *)); i++, stmt++) {
            if (*stmt) {
                sqlite3_finalize(*stmt);
                *stmt = NULL;
            }
        }

        if (client->dstore.handle) {
            sqlite3_close(client->dstore.handle);
            client->dstore.handle = NULL;
        }

        if (client->states & NMAP_CLIENT_MBOX) {
            client->states &= ~NMAP_CLIENT_MBOX;

            if (!(client->mail.flags & NMAP_SHARE_MAILBOX)) {
                sprintf(client->path, "%s/%s/%s.box", client->store, client->user, client->mail.name);
                if ((stat(client->path, &sb) == 0) && (client->mail.size != (unsigned long)sb.st_size)) {
                    ParseMailStore(client);                                                                 
                }

                if (client->mail.fh) {
                    fclose(client->mail.fh);
                    client->mail.fh = NULL;
                }

#if 0
                StoreMaildrop(client);
#endif
            } else {
#if 0
                StoreSharedCalendar(client);
#endif

                if (client->mail.share.conn) {
                    ConnClose(client->mail.share.conn, 0);
                    ConnFree(client->mail.share.conn);
                    client->mail.share.conn = NULL;
                }
            }
        }

        if (client->mail.lock) {
            ReadNLockRelease(client->mail.lock);
            client->mail.lock = NULL;
        }

        if (client->mail.info.list) {
            MemFree(client->mail.info.list);
            client->mail.info.list = NULL;
            client->mail.info.allocated = 0;
            client->mail.info.used = 0;
            client->mail.info.count = 0;
        }

        stmt = &(client->mail.info.sql.begin);
        for (i = 0; i < (sizeof(NmapClientMailStatements) / sizeof(sqlite3_stmt *)); i++, stmt++) {
            if (*stmt) {
                sqlite3_finalize(*stmt);
                *stmt = NULL;
            }
        }

        if (client->mail.info.handle) {
            sqlite3_close(client->mail.info.handle);
            client->mail.info.handle = NULL;
        }

        stmt = &(client->cal.info.sql.begin);
        for (i = 0; i < (sizeof(NmapClientCalStatements) / sizeof(sqlite3_stmt *)); i++, stmt++) {
            if (*stmt) {
                sqlite3_finalize(*stmt);
                *stmt = NULL;
            }
        }

        if (client->states & NMAP_CLIENT_CAL) {
            client->states &= ~NMAP_CLIENT_CAL;
        }

        ContactClose(client);

        if (client->conn) {
            ConnClose(client->conn, 0);
            ConnFree(client->conn);
            client->conn = NULL;
        }
    }

    return;
}

#define INDEX_CONNECT_TIMEOUT 500 /* in milliseconds */

__inline static Connection * 
ConnectToIndexer(void)
{
    Connection *conn;
    long ccode;

    unsigned char line[CONN_BUFSIZE + 1];
    BOOL result;

    conn = ConnAlloc(TRUE);
    if (conn) {

        memset(&conn->socketAddress, 0, sizeof(struct sockaddr_in));
        conn->socketAddress.sin_family = AF_INET;
        conn->socketAddress.sin_addr.s_addr = NMAP.indexConnection.ipAddress;
        conn->socketAddress.sin_port = NMAP.indexConnection.port;

        conn->socket = IPsocket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
        if (conn->socket != -1) {
            ccode = XplIPConnectWithTimeout(conn->socket, (struct sockaddr *)&(conn->socketAddress), sizeof(struct sockaddr_in), INDEX_CONNECT_TIMEOUT);

            if (!ccode) {
                /* Turn off Nagle */
                ccode = 1;
                setsockopt(conn->socket, IPPROTO_TCP, 1, (unsigned char *)&ccode, sizeof(ccode));
                result = NMAPAuthenticate(conn, line, CONN_BUFSIZE);

                if (result) {
                    return(conn);
                }
            }

            IPclose(conn->socket);
            conn->socket = -1;
        }

        ConnFree(conn);
    }

    return(NULL);
}

__inline static IndexConnection *
GetIndexConnection(void)
{
    IndexConnection *indexConn;

    XplWaitOnLocalSemaphore(NMAP.indexConnection.semaphore);
    if (NMAP.indexConnection.head) {
        indexConn = NMAP.indexConnection.head;
        NMAP.indexConnection.head = NMAP.indexConnection.head->next;
        NMAP.indexConnection.count--;
        XplSignalLocalSemaphore(NMAP.indexConnection.semaphore);
        return(indexConn);        
    }

    XplSignalLocalSemaphore(NMAP.indexConnection.semaphore);

    indexConn = MemMalloc(sizeof(IndexConnection));
    if (indexConn) {
        indexConn->cached = FALSE;
        indexConn->conn = ConnectToIndexer();
        if (indexConn->conn) {
            return(indexConn);
        }

        MemFree(indexConn);
    }

    return(NULL);
}

__inline static void
DestroyIndexConnection(IndexConnection *indexConn)
{
    ConnFree(indexConn->conn);
    MemFree(indexConn);
}

__inline static void
ReleaseIndexConnection(IndexConnection *indexConn)
{
    XplWaitOnLocalSemaphore(NMAP.indexConnection.semaphore);
    if (NMAP.indexConnection.count < NMAP.indexConnection.limit) {
        indexConn->cached = TRUE;
        indexConn->next = NMAP.indexConnection.head;
        NMAP.indexConnection.head = indexConn;
        NMAP.indexConnection.count++;
        XplSignalLocalSemaphore(NMAP.indexConnection.semaphore);
        return;
    }

    XplSignalLocalSemaphore(NMAP.indexConnection.semaphore);

    DestroyIndexConnection(indexConn);
    return;
}

BOOL
NotifyIndexer(unsigned char *user)
{
    IndexConnection *indexConn;

    do {
        indexConn = GetIndexConnection();
        if (indexConn) {
            Connection *conn = indexConn->conn;
            unsigned char buffer[10];
            if (ConnWriteF(conn, "INDEX %s NMAP NonInteractive\r\n", user) != -1) {
                if (ConnFlush(conn) != -1) {
                    if (ConnReadAnswer(conn, buffer, sizeof(buffer)) != -1) {
                        if (strcmp(buffer, "OK") == 0) {
                            ReleaseIndexConnection(indexConn);
                            return(TRUE);
                        }
                    }
                }
            }

            ConnClose(conn, 0);

            if (indexConn->cached) {
                /* we need to keep trying just in case a connection went bad in the cache */
                DestroyIndexConnection(indexConn);
                continue;
            }
            DestroyIndexConnection(indexConn);
        }

        break;
    } while(TRUE);

    return(FALSE);
}

int 
NmapCommandAuth(void *param)
{
    int i;
    unsigned char *ptr;
    unsigned char digest[16];
    unsigned char credential[33];
    MD5_CTX mdContext;
    NMAPClient *client = (NMAPClient *)param;

    if (!client->states) {
        if (client->buffer[4] == ' ') {
            ptr = client->buffer + 5;
        } else {
            return(ConnWrite(client->conn, MSG3010BADARGC, sizeof(MSG3010BADARGC) - 1));
        }
    } else {
        return(ConnWrite(client->conn, MSG3240BADSTATE, sizeof(MSG3240BADSTATE) - 1));
    }

    MD5_Init(&mdContext);
    MD5_Update(&mdContext, client->user, strlen(client->user));
    MD5_Update(&mdContext, NMAP.server.hash, NMAP_HASH_SIZE);
    MD5_Final(digest, &mdContext);

    for (i = 0; i < 16; i++) {
        sprintf(credential + (i * 2), "%02x", digest[i]);
    }

    if (QuickCmp(credential, ptr)) {
        client->user[0] = '\0';
        client->states |= NMAP_CLIENT_AUTHENTICATED;
        return(ConnWrite(client->conn, MSG1000OK, sizeof(MSG1000OK) - 1));
    }

    LoggerEvent(NMAP.handle.logging, LOGGER_SUBSYSTEM_AUTH, LOGGER_EVENT_WRONG_SYSTEM_AUTH, LOG_NOTICE, 0, NULL, NULL, XplHostToLittle(client->conn->socketAddress.sin_addr.s_addr), 0, NULL, 0);

    ConnWrite(client->conn, MSG3242BADAUTH, sizeof(MSG3242BADAUTH) - 1);

    XplDelay(2000);

    FreeClientData(client);

    return(-1);
}

__inline static int 
ConfigureUser(NMAPClient *client)
{
    int ccode;
    unsigned char *ptr;
    struct stat sb;
    FILE *handle;
    MDBValueStruct *vs;

    client->store = MsgFindUserStore(client->user, NMAP.path.mail);

#if defined(NETWARE) || defined(LIBC)
    sprintf(client->line, "NMAP: %s", client->user);
    XplRenameThread(XplGetThreadID(), client->line);
#endif

    ptr = client->path;
    ptr += sprintf(client->path, "%s/%s/", client->store, client->user);

    memcpy(ptr, "INBOX.box", 10);
    if (stat(client->path, &sb) == 0) {
        memcpy(ptr, "MAIN.cal", 9);
    } else {
        ptr[-1] = '\0';
        XplMakeDir(client->path);

        memcpy(ptr - 1, "/INBOX.box", 11);
        handle = fopen(client->path, "wb");
        if (handle) {
            fclose(handle);

            memcpy(ptr, "INBOX.idx", 10);
            handle = fopen(client->path, "wb");
            fprintf(handle, "%s\r\n0000000000\r\n0000000000\r\n0000000000\r\n%010lu\r\n%010lu\r\n", NMAP_IDX_VERSION, NMAP.universalCounter++, NMAP.universalCounter++);
            fclose(handle);

            sprintf(client->line, "%lu", NMAP.universalCounter++);
            vs = MDBCreateValueStruct(NMAP.handle.directory, NMAP.server.dn);
            MDBAddValue(client->line, vs);
            MDBWrite(MSGSRV_AGENT_NMAP, MSGSRV_A_UID, vs);
            MDBDestroyValueStruct(vs);
        }

        memcpy(ptr, "MAIN.cal", 9);
    }

    if (stat(client->path, &sb) == 0) {
        memcpy(ptr, "Personal.cav", 13);
    } else {
        handle = fopen(client->path, "wb");
        if (handle != NULL) {
            fclose(handle);

            memcpy(ptr, "MAIN.idc", 9);
            handle = fopen(client->path, "wb");
            fprintf(handle, "%s\r\n0000000000\r\n0000000000\r\n0000000000\r\n%010lu\r\n%010lu\r\n", NMAP_CAL_IDX_VERSION, NMAP.universalCounter++, NMAP.universalCounter++);
            fclose(handle);

            sprintf(client->line, "%lu", NMAP.universalCounter);
            vs = MDBCreateValueStruct(NMAP.handle.directory, NMAP.server.dn);
            MDBAddValue(client->line, vs);
            MDBWrite(MSGSRV_AGENT_NMAP, MSGSRV_A_UID, vs);
            MDBDestroyValueStruct(vs);
        }

        memcpy(ptr, "Personal.cav", 13);
    }

    if (stat(client->path, &sb) != 0) {
        handle = fopen(client->path, "wb");
        if (handle != NULL) {
            fclose(handle);
        }
    }

    if ((ccode = DStoreOpen(client)) == SQLITE_DONE) {
        client->states |= NMAP_CLIENT_USER;

        return(0);
    }

    return(-1);
}

int 
NmapCommandPass(void *param)
{
    unsigned char *ptr;
    unsigned char *pass;
    BOOL result;
    struct in_addr nmap;
    MDBValueStruct *vs;
    NMAPClient *client = (NMAPClient *)param;

    if (!client->states) {
        ptr = client->buffer + 4;
    } else {
        return(ConnWrite(client->conn, MSG3240BADSTATE, sizeof(MSG3240BADSTATE) - 1));
    }

    if (*ptr++ == ' ') {
        if (       (toupper(ptr[0]) == 'U') 
                && (toupper(ptr[1]) == 'S') 
                && (toupper(ptr[2]) == 'E') 
                && (toupper(ptr[3]) == 'R') 
                && (ptr[4] == ' ') 
                && (ptr[5] != '\0') 
                && (!isspace(ptr[5])) 
                && ((pass = strchr(ptr + 5, ' ')) != NULL)) {
            ptr += 5;
            *pass++ = '\0';

            vs = MDBCreateValueStruct(NMAP.handle.directory, NULL);
            if (vs) {
                result = MsgFindObject(ptr, client->dn, NULL, &nmap, vs);
            } else {
                return(ConnWrite(client->conn, MSG5001NOMEMORY, sizeof(MSG5001NOMEMORY) - 1));
            }

            if (result) {
                result = MDBVerifyPassword(client->dn, pass, vs);
            }

            if (result) {
                strcpy(client->user, vs->Value[0]);
            }

            MDBDestroyValueStruct(vs);

            if (result && (nmap.s_addr == MsgGetHostIPAddress())) {
                GET_STRING_HASH(client->user, client->userHash);

                if (ConfigureUser(client) == 0) {
                    client->flags |= NMAP_KEEPUSER;
                    return(ConnWrite(client->conn, MSG1000OK, sizeof(MSG1000OK) - 1));
                }

                return(ConnWrite(client->conn, MSG4120USERLOCKED, sizeof(MSG4120USERLOCKED) - 1));
            }

            LoggerEvent(NMAP.handle.logging, LOGGER_SUBSYSTEM_AUTH, LOGGER_EVENT_WRONG_USER_AUTH, LOG_NOTICE, 0, ptr, NULL, XplHostToLittle(client->conn->socketAddress.sin_addr.s_addr), 0, NULL, 0);
        } else if (    (toupper(ptr[0]) == 'S') 
                    && (toupper(ptr[1]) == 'Y') 
                    && (toupper(ptr[2]) == 'S') 
                    && (toupper(ptr[3]) == 'T') 
                    && (toupper(ptr[4]) == 'E') 
                    && (toupper(ptr[5]) == 'M') 
                    && (ptr[6] == ' ')) {
            ptr += 7;

#if defined(DEBUG)
            if (strcmp(NMAP.server.hash, ptr) == 0) {
                client->states |= NMAP_CLIENT_AUTHENTICATED;

                return(ConnWrite(client->conn, MSG1000OK, sizeof(MSG1000OK) - 1));
            }
#endif

            LoggerEvent(NMAP.handle.logging, LOGGER_SUBSYSTEM_AUTH, LOGGER_EVENT_WRONG_SYSTEM_AUTH, LOG_NOTICE, 0, NULL, NULL, XplHostToLittle(client->conn->socketAddress.sin_addr.s_addr), 0, NULL, 0);
        }
    }

    XplDelay(2000);

    ConnWrite(client->conn, MSG3010BADARGC, sizeof(MSG3010BADARGC) - 1);

    FreeClientData(client);
    return(-1);
}


int
NmapCommandPrinc(void *param)
{
    unsigned char *ptr;
    NMAPClient *client = (NMAPClient *)param;
    NMAPPrincipalType type;

    /* PRINC {USER | EMAIL} <principal> */

    if (!(client->states & NMAP_CLIENT_AUTHENTICATED)) {
        return ConnWrite(client->conn, MSG3240NOAUTH, sizeof(MSG3240NOAUTH) - 1);
    }
    if (client->flags & NMAP_KEEPUSER) {
        return ConnWrite(client->conn, MSG4240NOPERMISSION, sizeof(MSG4240NOPERMISSION));
    }

    ptr = client->buffer + 5;
    if (!XplStrNCaseCmp(" USER ", ptr, 6)) {
        type = NMAP_PRINCIPAL_USER;
        ptr += 6;
    } else if (!XplStrNCaseCmp(" EMAIL ", ptr, 7)) {
        type = NMAP_PRINCIPAL_EMAIL;
        ptr += 7;
    } else {
        return ConnWrite(client->conn, MSG3010BADARGC, sizeof(MSG3010BADARGC) - 1);
    }
    if (!*ptr || isspace(*ptr)) {
        return ConnWrite(client->conn, MSG3010BADARGC, sizeof(MSG3010BADARGC) - 1);
    }

    strncpy(client->principal, ptr, sizeof(client->principal));
    client->principal[sizeof(client->principal)-1] = 0;
    client->principalType = type;

    return ConnWrite(client->conn, MSG1000OK, sizeof(MSG1000OK) - 1);
}


int 
NmapCommandUlist(void *param)
{
    int ccode;
    unsigned long used;
    const unsigned char *name;
    const unsigned char *ptr;
    NMAPClient *client = (NMAPClient *)param;
    MDBValueStruct *vs;
    MDBEnumStruct *es;

    /* fixme - verify command buffer; no arguments means no delimiter. */
    /* fixme - add a 'LOCAL' argument. */
    if (client->states & NMAP_CLIENT_AUTHENTICATED) {
        vs = MDBCreateValueStruct(NMAP.handle.directory, NULL);
    } else {
        return(ConnWrite(client->conn, MSG3240NOAUTH, sizeof(MSG3240NOAUTH) - 1));
    }

    if (vs) {
        es = MDBCreateEnumStruct(vs);
    } else {
        return(ConnWrite(client->conn, MSG5001NOMEMORY, sizeof(MSG5001NOMEMORY) - 1));
    }

    if (es) {
        MDBReadDN(NMAP.server.dn, MSGSRV_A_CONTEXT, vs);
    } else {
        MDBDestroyValueStruct(vs);

        return(ConnWrite(client->conn, MSG5001NOMEMORY, sizeof(MSG5001NOMEMORY) - 1));
    }

    ccode = 0;
    for (used = 0; (ccode != -1) && (used < vs->Used); used++) {
        while ((name = MDBEnumerateObjectsEx(vs->Value[used], C_USER, NULL, FALSE, es, vs)) != NULL) {
            if ((ptr = strrchr(name, '\\')) != NULL) {
                ccode = ConnWriteF(client->conn, "2001-%s\r\n", ptr + 1);
            } else {
                ccode = ConnWriteF(client->conn, "2001-%s\r\n", name);
            }
        }
    }

    MDBDestroyEnumStruct(es, vs);
    MDBDestroyValueStruct(vs);

    if (ccode != -1) {
        ccode = ConnWrite(client->conn, MSG1000OK, sizeof(MSG1000OK) - 1);
    }

    return(ccode);
}

int 
NmapCommandUser(void *param)
{
    unsigned long flags = 0;
    unsigned char *ptr;
    unsigned char *ptr2;
    NMAPClient *client = (NMAPClient *)param;

    if (client->states & NMAP_CLIENT_AUTHENTICATED) {
        if (!(client->states & NMAP_CLIENT_USER)) {
            ptr = client->buffer + 4;
        } else {
            return(ConnWrite(client->conn, MSG4221USERDONE, sizeof(MSG4221USERDONE) - 1));
        }
    } else {
        return(ConnWrite(client->conn, MSG3240NOAUTH, sizeof(MSG3240NOAUTH) - 1));
    }

    if ((*ptr++ == ' ') && (!isspace(*ptr))) {
        ptr2 = strchr(ptr, ' ');
        if (ptr2) {
            *ptr2++ = '\0';
            flags = atol(ptr2);
        }
    } else {
        return(ConnWrite(client->conn, MSG3010BADARGC, sizeof(MSG3010BADARGC) - 1));
    }

    strcpy(client->user, ptr);

    GET_STRING_HASH(ptr, client->userHash);

#if 0
    /* Check if user exists ...

    if (User doesn't exist) {
        ConnWrite(client->conn, MSG4224NOMBOX, sizeof(MSG4224NOMBOX) - 1);
        break;
    } */
#endif

    if (ConfigureUser(client) == 0) {
        return(ConnWrite(client->conn, MSG1000USER, sizeof(MSG1000USER) - 1));
    }

    return(ConnWrite(client->conn, MSG4120USERLOCKED, sizeof(MSG4120USERLOCKED) - 1));
}

int 
NmapCommandManage(void *param)
{
    int ccode;
    unsigned long serverNum;
    MDBValueStruct *vs;
    NMAPClient *client = (NMAPClient *)param;
    char *ptr;

    if (client->states && NMAP_CLIENT_USER) {
        vs = MDBCreateValueStruct(NMAP.handle.directory, NULL);
        if (vs) {
            if (MDBReadDN(NMAP.server.dn, MSGSRV_A_MANAGER, vs)) {
                for (serverNum = 0; serverNum < vs->Used; serverNum++) {
                    ptr = strrchr(vs->Value[serverNum], '\\');
                    if (ptr) {
                        ++ptr;
                    } else {
                        ptr = vs->Value[serverNum];
                    }
                    if (XplStrCaseCmp(client->user, ptr) == 0) {
                        client->flags &= ~NMAP_KEEPUSER;
                        client->states |= NMAP_CLIENT_AUTHENTICATED;

                        ccode = ConnWrite(client->conn, MSG1000OK, sizeof(MSG1000OK) - 1);
                        MDBDestroyValueStruct(vs);
                        return(ccode);
                    }
                }
            }

            ccode = ConnWrite(client->conn, DMCMSG2001NOT_MANAGER, sizeof(DMCMSG2001NOT_MANAGER) - 1);
            MDBDestroyValueStruct(vs);
        } else {
            ccode = ConnWrite(client->conn, MSG5001NOMEMORY, sizeof(MSG5001NOMEMORY) - 1);
        }
    } else {
        ccode = ConnWrite(client->conn, MSG3240BADSTATE, sizeof(MSG3240BADSTATE) - 1);
    }

    return(ccode);
}
