/****************************************************************************
 * <Novell-copyright>
 * Copyright (c) 2001 Novell, Inc. All Rights Reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, contact Novell, Inc.
 * 
 * To contact Novell about this file by physical or electronic mail, you 
 * may find current contact information at www.novell.com.
 * </Novell-copyright>
 ****************************************************************************/

#include <config.h>
#include <xpl.h>
#include <memmgr.h>

#include <calcmd.h>




static char *Months[12] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                            "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

static char *WeekDays[7] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };

static int
PrintConciseDateTimeTM(struct tm *when, char *buffer, size_t size)
{
    time_t tnow, twhen;
    struct tm now;
    char tmbuf[10];
    

    time(&tnow);
    localtime_r(&tnow, &now);
    twhen = mktime(when);

    if (when->tm_hour >= 12) {
        if (when->tm_min) {
            sprintf (tmbuf, "%d:%02dPM", 
                     12 == when->tm_hour ? 12 : when->tm_hour - 12, 
                     when->tm_min);
        } else {
            sprintf (tmbuf, "%dPM", 
                     12 == when->tm_hour ? 12 : when->tm_hour - 12);
        }
    } else {
        if (when->tm_min) {
            sprintf (tmbuf, "%d:%02dAM", 
                     0 == when->tm_hour ? 12 : when->tm_hour, 
                     when->tm_min);
        } else {
            sprintf (tmbuf, "%dAM", 
                     0 == when->tm_hour ? 12 : when->tm_hour);
        }       
    }


    if (now.tm_year != when->tm_year) {
        /* full */
        return snprintf(buffer, size, "%s %s %d %4d %s",
                        WeekDays[when->tm_wday], Months[when->tm_mon], 
                        when->tm_mday, when->tm_year + 1900,
                        tmbuf);
    } else if (twhen - tnow < 60 * 60 * 24 * 6 && tnow < twhen) {
        return snprintf(buffer, size, "%s %s",
                        WeekDays[when->tm_wday], tmbuf);
    } else {
        return snprintf(buffer, size, "%s %s %d %s",
                        WeekDays[when->tm_wday], Months[when->tm_mon], 
                        when->tm_mday, tmbuf);
    }

}


/* It's the date time printout that's concise, not the name of this function :-) */

int
CalCmdPrintConciseDateTimeRangeTM(struct tm *begin, struct tm *end, 
                            char *buffer, size_t size)
{
    int len = 0;

    len += PrintConciseDateTimeTM(begin, buffer, size);
    if ((unsigned int)len >= size) {
        buffer[size-1] = 0;
        return size;
    }
/* FIXME: make this code more intelligent.  i.e. print "fri 2-3pm" instead
   of the wordy "fri 2pm - fri 3pm"

    len += snprintf(buffer + len, size - len, " - ");
    if ((unsigned int)len >= size) {
        buffer[size-1] = 0;
        return size;
    }
    
    len += PrintConciseDateTimeTM(end, buffer + len, size - len);
    if ((unsigned int)len >= size) {
        buffer[size-1] = 0;
        return size;
    }
*/  
    return len;
}




/* Convert local time to UTC.  These should eventually be replaced
 * with something that uses the user's default timezone rather than
 * the machine-local time zone */
void
CalCmdLocalToUtc(struct tm *val, struct tm *valOut)
{
    time_t begin_time = mktime(val);
    gmtime_r(&begin_time, valOut);
}

void
CalCmdUtcToLocal(struct tm *val, struct tm *valOut)
{
    time_t begin_time = mktime(val);
    
    localtime_r(&begin_time, valOut);
    begin_time += valOut->tm_gmtoff;
    localtime_r(&begin_time, valOut);
}


int
CalCmdProcessNewAppt(CalCmdCommand *cmd, char *icalbuf, size_t size, char *uid)
{
    int len;
    struct tm begin;
    struct tm end;


    fprintf(stderr, "processnewAppt(): %d uid: %s\r\n", cmd->type, uid);
/*

    if (cmd->type != CALCMD_NEW_APPT) {
        return 0;
    }
*/
    /* Convert to UTC */
    /* FIXME: Include the user's default time zone and don't use utc times */
    CalCmdLocalToUtc(&cmd->begin, &begin);
    CalCmdLocalToUtc(&cmd->end, &end);

    len = 
    snprintf(icalbuf, size,
             
             "BEGIN:VCALENDAR\r\n"
             "BEGIN:VEVENT\r\n"
             "SUMMARY:%s\r\n"
             "DTSTART:%04d%02d%02dT%02d%02d%02dZ\r\n"
             "DTEND:%04d%02d%02dT%02d%02d%02dZ\r\n"
             "UID:%s\r\n"
             "END:VEVENT\r\n"
             "END:VCALENDAR\r\n",
             
             cmd->data,
             
             begin.tm_year + 1900, begin.tm_mon + 1, begin.tm_mday,
             begin.tm_hour, begin.tm_min, begin.tm_sec,
             
             end.tm_year + 1900, end.tm_mon + 1, end.tm_mday,
             end.tm_hour, end.tm_min, end.tm_sec,

             uid
        );
    icalbuf[size-1] = 0;

    fprintf (stderr, "result (%d):\r\n%s\r\n", len, icalbuf);

    return len;
}


/* these functions provided for the benefit of mono code */


int32_t
CalCmdGetType(CalCmdCommand *cmd) 
{
    return cmd->type;
}


void 
CalCmdSetType(CalCmdCommand *cmd, int32_t value)
{
    cmd->type = value;
}


char *
CalCmdGetData(CalCmdCommand *cmd)
{
    return cmd->data;
}

void 
CalCmdSetData(CalCmdCommand *cmd, char *data)
{
    strncpy (cmd->data, data, sizeof(cmd->data));
    cmd->data[sizeof(cmd->data)-1] = 0;
}


int64_t
CalCmdGetBegin(CalCmdCommand *cmd)
{
    return mktime (&cmd->begin);
}


int64_t
CalCmdGetEnd(CalCmdCommand *cmd)
{
    return mktime (&cmd->end);
}


int
CalCmdPrintConciseDateTimeRange(int64_t beginMillisLocal, int64_t endMillisLocal, 
                                char *buffer, size_t size)
{
    struct tm begin;
    struct tm end;

    beginMillisLocal /= 1000;
    endMillisLocal /= 1000;

    localtime_r(&beginMillisLocal, &begin);
    localtime_r(&endMillisLocal, &end);

    return CalCmdPrintConciseDateTimeRangeTM(&begin, &end, buffer, size);
}
