/****************************************************************************
 *
 * Copyright (c) 2005 Novell, Inc.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, contact Novell, Inc.
 *
 * To contact Novell about this file by physical or electronic mail,
 * you may find current contact information at www.novell.com
 *
 ****************************************************************************/

using System;
using System.Collections;
using System.IO;

using Hula.Store;

namespace Hula.Importing {

public abstract class Importer
{
    private static Hashtable extImporters = null;
    private static Hashtable typeImporters = null;

    private static void InitImporters ()
    {
	extImporters = new Hashtable ();
	typeImporters = new Hashtable ();

	extImporters[".vcf"] = typeof (VCardImporter);
	typeImporters["text/x-vcard"] = typeof (VCardImporter);

#if true
	extImporters[".ics"] = typeof (ICalendarImporter);
	typeImporters["text/calendar"] = typeof (ICalendarImporter);
#endif
    }

    public static Importer GetImporter (string filename,
					string contentType,
					Stream stream)
    {
	Type t = null;

	object[] args = new object [3];
	args[0] = filename;
	args[1] = contentType;
	args[2] = stream;

	if (extImporters == null) {
	    InitImporters ();
	}

	t = (Type)typeImporters[contentType];
	if (t == null) {
	    int dotIdx = filename.LastIndexOf ('.');
	    if (dotIdx != -1) {
		t = (Type)extImporters[filename.Substring (dotIdx)];
	    }
	}
	return t != null ? (Importer)Activator.CreateInstance (t, args) : null;
    }

    public abstract IList ImportTo (IStore store, string folder);
}

}
