using System.Collections;
using System.IO;
using Lucene.Net.Index;
using Lucene.Net.Documents;
using Hula.Store;

namespace Hula.Indexing {

public abstract class Indexer {
    static Indexer() {
	AddIndexer("text/calendar", new Hula.Indexing.CalIndexer());
	AddIndexer("text/x-vcard", new Hula.Indexing.VCardIndexer());
	AddIndexer("application/x-directory", new Hula.Indexing.FolderIndexer());
    }

    public static Indexer GetIndexer(string mimeType) {
	if (filters.Contains(mimeType)) {
	    return (Indexer)filters[mimeType];
	} else {
	    return genericIndexer;
	}
    }

    private static Hashtable filters = new Hashtable();
    private static Indexer genericIndexer = new GenericIndexer();

    public static void AddIndexer(string mimeType, Indexer filter) {
	filters[mimeType] = filter;
    }

    public abstract void IndexDocument(LuceneIndex index,
				       IStore store,
				       string guid);

    protected void GetDocument(IStore store, string guid, out TextReader reader, out IDictionary properties) {
	store.Get(guid, out properties, out reader);
    }
    
    protected void AddProperties(Document doc, IDictionary properties) {
	if (properties != null) {
	    foreach(string key in properties.Keys) {
                string value = (string)properties[key];
                doc.Add(Field.Text(key, value));
	    }
	}
    }

    protected void AddContentFields(Document doc, IDictionary properties,
                                    IDictionary everythingScores) {
        if (properties != null) {
            foreach(string key in properties.Keys) {
                string value = (string)properties[key];
                doc.Add(Field.Text(key, value));

                if (everythingScores.Contains(key))
                {
                    Field field = Field.UnStored("everything", value);
                    if (everythingScores[key] != null) {
                        field.SetBoost((float)everythingScores[key]);
                    }
                    doc.Add(field);
                }
            }
        }
    }
    
    protected void AddStandardFields(Document doc, IStore store, string guid, IDictionary properties) {
	doc.Add(Field.Keyword("guid", guid));

	string folder = (string)properties["hula.folder"];
	string filename = "";
	if (properties.Contains("hula.davfilename")) {
	    filename = (string)properties["hula.davfilename"];
	} else {
	    filename = guid;
	}

	string path;
	if (folder == "/") {
	    path = "/" + filename;
	} else {
	    path = folder + "/" + filename;
	}
	
	doc.Add(Field.Keyword("hula.davhref", "/" + store.Owner + "/" + store.Name + path));
	doc.Add(Field.Keyword("hula.owner", store.Owner));
	doc.Add(Field.Keyword("hula.davpath", path));
    }
}
}
