using System;
using System.IO;
using System.Text;
using System.Collections;
using System.Xml;
using Lucene.Net;
using Lucene.Net.Analysis.Standard;
using Lucene.Net.Documents;
using Lucene.Net.Index;
using Lucene.Net.Store;
using Hula.Store;

namespace Hula.Indexing {
public class VCardIndexer : Indexer {
    private static Hashtable indexFields = new Hashtable();
    private static Hashtable bodyFields = new Hashtable();
    private static Hashtable everythingScores = new Hashtable();

    static VCardIndexer() {
	indexFields["NICKNAME"] = true;
	indexFields["EMAIL"] = true;
	indexFields["FN"] = true;
	indexFields["LABEL"] = true; 

	bodyFields["URL"] = true;
	bodyFields["TITLE"] = true;
	bodyFields["ORG"] = true; 
	bodyFields["ROLE"] = true;
	bodyFields["NICKNAME"] = true;
	bodyFields["EMAIL"] = true;
	bodyFields["FN"] = true;
	bodyFields["LABEL"] = true;

        everythingScores["URL"] = null;
        everythingScores["TITLE"] = null;
        everythingScores["ORG"] = null;
        everythingScores["ROLE"] = null;
        everythingScores["NICKNAME"] = null;
        everythingScores["EMAIL"] = null;
        everythingScores["FN"] = null;
        everythingScores["LABEL"] = null;
    }

    public VCardIndexer() : base(){
    }

    private static string ReadFoldedLine(TextReader reader) {
	string line = reader.ReadLine();
	if (line == null) {
	    return null;
	}
	line = line.Replace("\\n", "\n");
	int c = reader.Peek();
	if (c == ' ') {
	    StringBuilder builder = new StringBuilder();
	    builder.Append(line);
	    do {
		string newLine = reader.ReadLine();
		if (newLine == null) {
		    break;
		}
		newLine = newLine.Replace("\\n", "\n");
		    
		builder.Append(newLine.Substring(1));
		c = reader.Peek();
	    } while (c == ' ');
	    return builder.ToString();		
	} else {
	    return line;
	}
    }

    public override void IndexDocument(LuceneIndex idx,
				       IStore store,
				       string guid) {
	TextReader reader;
	IDictionary properties;
	GetDocument(store, guid, out reader, out properties);
	using(reader) {
            IDictionary contentFields = new Hashtable();

	    Document doc = new Document();
	    
	    string line = ReadFoldedLine(reader);
	    while (line != null) {
		if (!(line == "" || line.StartsWith("BEGIN:") || line.EndsWith("END:"))) {
		    AddLine(line, doc, contentFields);
		}
		line = ReadFoldedLine(reader);
	    }
	    
	    AddProperties(doc, properties);
	    AddStandardFields(doc, store, guid, properties);
	    AddContentFields(doc, contentFields, everythingScores);
	    
	    idx.Writer.AddDocument(doc);
	}
    }

    public struct VCardLine {
	internal static char[] colon = ":".ToCharArray();
	internal static char[] semiColon = ";".ToCharArray();

	public string key;
	public string parameters;
	public string value;
	
	public static VCardLine Parse(string line) {
	    VCardLine ret = new VCardLine();
	    string[] lines = line.Split(colon, 2);
	    string[] linesParam = lines[0].Split(semiColon, 2);
	    
	    ret.key = linesParam[0];
            if (lines.Length > 1) {
                ret.value = lines[1];
            } else {
                ret.value = "";
            }
	    if (linesParam.Length > 1) {
		ret.parameters = linesParam[1];
	    } else {
		ret.parameters = "";
	    }
	    
	    return ret;
	}
    };

    private void AddLine(string lineStr, Document doc, IDictionary contentFields) { 
	// Console.Error.WriteLine ("Parsing line: {0}", lineStr);
	VCardLine line = VCardLine.Parse(lineStr);
	if (bodyFields.Contains(line.key)) {
	    contentFields[line.key] = line.value;
	} 

	if (indexFields.Contains(line.key)) {
	    string key = "vcard." + line.key.ToLower();
	    doc.Add(Field.Text(key, line.value));
	    
	    doc.Add(Field.Keyword(key + ".value", line.value));
	    doc.Add(Field.Keyword(key + ".params", line.parameters));
	}
    }
}
}
