/****************************************************************************
 *
 * Copyright (c) 2001-2002 Novell, Inc.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, contact Novell, Inc.
 *
 * To contact Novell about this file by physical or electronic mail,
 * you may find current contact information at www.novell.com
 *
 ****************************************************************************/

using System;
using System.Collections;

using log4net;

namespace Hula.Nmap
{
public class NmapException : Exception
{
    private static readonly ILog log = LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

    private static Hashtable codes = null;

    static NmapException()
    {
        if (codes == null)
        {
            log.Debug("initializing NmapException");
            codes = new Hashtable();
            codes.Add("1000", "Success");
            codes.Add("2000", "Success, info follows");
            codes.Add("2001", "Success, info in Success-message");
            codes.Add("2002", "Success, info in Success-message and multi-line response");
            codes.Add("2003", "Multipart end");
            codes.Add("2004", "rfc822 end");
            codes.Add("2020", "Header follows");
            codes.Add("2021", "Body follows");
            codes.Add("2022", "Info follows");
            codes.Add("2023", "Header and Body follows");
            codes.Add("2024", "Header and Non-Escaped Body follows");
            codes.Add("2025", "Non-Escaped Body follows");
            codes.Add("2026", "Ready to receive header");
            codes.Add("2027", "Ready to receive body");
            codes.Add("2028", "Ready to receive complete message");
            codes.Add("2029", "Ready to receive recipients");
            codes.Add("2050", "Send Message");
            codes.Add("2051", "Send Info");
            codes.Add("2052", "Send Sender");
            codes.Add("2053", "Send Recipient(s)");
            codes.Add("2054", "Send Flags");
            codes.Add("3000", "Command unknown");
            codes.Add("3010", "Wrong number of arguments");
            codes.Add("3011", "Argument <%d> wrong format");
            codes.Add("3012", "Wrong state, not waiting on queues");
            codes.Add("3013", "Wrong state, waiting on queues");
            codes.Add("3240", "Command issued in wrong state");
            codes.Add("3241", "No user selected, try USER");
            codes.Add("4000", "Argument <%d> out of range");
            codes.Add("4020", "String not found");
            codes.Add("4120", "Mailbox locked");
            codes.Add("4220", "Message <%d> out of range");
            codes.Add("4221", "Mailbox already locked, try later; Mailbox already selected; User already selected");
            codes.Add("4222", "Message is already deleted");
            codes.Add("4223", "Message was not marked deleted");
            codes.Add("4224", "Mailbox does not exist; Queue entry does not exist; Cannot read queue data; Cannot write queue data; Cannot read mailbox");
            codes.Add("4225", "No mailbox selected");
            codes.Add("4226", "Mailbox already exists; Queue entry still open");
            codes.Add("4227", "Mailbox is currently selected");
            codes.Add("4228", "Cannot write to mailbox");
            codes.Add("4229", "Cannot write to message store");
            codes.Add("4240", "Permission denied");
            codes.Add("4241", "Already authenticated");
            codes.Add("4260", "No queue entry open, try QCREA");
            codes.Add("4261", "No queue entry with that domain");
            codes.Add("4930", "Not enough memory");
            codes.Add("4931", "Not enough diskspace");
            codes.Add("5000", "Error requested");
            codes.Add("5001", "Out of memory");
            codes.Add("5002", "Server shutting down");
            codes.Add("5003", "Input line too long, shutting down.");
            codes.Add("5004", "Internal Error");
            codes.Add("5220", "Quota Exceeded");
            codes.Add("5221", "Free disk space too low");
            codes.Add("5900", "Internal server error, will shutdown");
            codes.Add("6000", "Mailbox changed [Format: 6000 <no-of-msgs> <no-of-new-msgs>]");
            codes.Add("6020", "Entry in queue now available");
            codes.Add("6021", "Please handle queue entry2003Multipart end");
        }
    }

    private int code;
    private string message;

    public NmapException(int code, string message) {
	this.code = code;
	this.message = message;
    }

    public NmapException(NmapResponse resp) {
	this.code = resp.code;
	this.message = resp.text;
    }

    public int Code {
	get { return code; }
    }

    public override string Message {
	get { return message; }
    }
}
}
