/****************************************************************************
 *
 * Copyright (c) 2005 Novell, Inc.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, contact Novell, Inc.
 *
 * To contact Novell about this file by physical or electronic mail,
 * you may find current contact information at www.novell.com
 *
 ****************************************************************************/

using System;
using System.Collections;
using log4net;

namespace Hula.Nmap
{
public enum RsetFlag
{
    User,
    Mbox,
    Cal
}

public class OpenCommands : NmapCommands
{
    public OpenCommands(NmapConnection nconn) : base(nconn)
    {
    }

    public string NVER()
    {
        nconn.WriteRequest("NVER");
        NmapResponse response = nconn.ReadFullReply ();
        return response.text;
    }

    public string ERRO()
    {
        nconn.WriteRequest("ERRO");
        NmapResponse response = nconn.ReadFullReply ();
        return response.text;
    }

    public IList ULIST()
    {
        nconn.WriteRequest("ULIST");
        NmapResponse response = nconn.ReadLongReply ();

        if (response.data == null) {
            return new ArrayList();
        } else {
            return new ArrayList(response.data.Split('\n'));
        }
    }

    public IList CAPA()
    {
        nconn.WriteRequest("CAPA");
        NmapResponse response = nconn.ReadLongReply ();
        return new ArrayList(response.data.Split('\n'));
    }

    public IDictionary CONF()
    {
        nconn.WriteRequest("CONF");
        NmapResponse response = nconn.ReadLongReply ();

        string[] lines = response.data.Split('\n');

        IDictionary conf = new Hashtable();

        for (int i=0; i<lines.Length; i++)
        {
            string[] keyval = lines[i].Split(new char[]{':'}, 2);

            if (!conf.Contains(keyval[0]))
            {
                conf.Add(keyval[0], keyval[1]);
            }
        }

        return conf;
    }

    public string RSET(RsetFlag flag)
    {
        string command = null;

        if (flag == RsetFlag.User)
        {
            command = "RSET";
        }
        else if (flag == RsetFlag.Mbox)
        {
            command = "RSET MBOX";
        }
        else if (flag == RsetFlag.Cal)
        {
            command = "RSET CAL";
        }

        nconn.WriteRequest(command);
        NmapResponse response = nconn.ReadFullReply ();
        return response.text;
    }
}
}
