using Lucene.Net;
using Lucene.Net.Analysis.Standard;
using Lucene.Net.Documents;
using Lucene.Net.Index;
using Lucene.Net.Search;
using Lucene.Net.Store;

namespace Hula.Util {

    public class SubstringTermEnum : FilteredTermEnum {
	private Term searchTerm;
	private string field = "";
	private string originalText = "";
	private string text = "";
	bool endEnum = false;
	bool caseSensitive;
	
	public SubstringTermEnum () {
	}

	public SubstringTermEnum (IndexReader reader, Term term, bool caseSensitive) : base() {
	    searchTerm = term;
	    field = searchTerm.Field();
	    originalText = searchTerm.Text();
	    if (caseSensitive) {
		text = originalText;
	    } else {
		text = originalText.ToLower();
	    }
	    this.caseSensitive = caseSensitive;

	    SetEnum(reader.Terms(new Term(field, "")));
	}

	public bool IsCaseSensitive {
	    get {
		return caseSensitive;
	    }
	    set {
		caseSensitive = value;
		if (caseSensitive) {
		    text = originalText;
		} else {
		    text = originalText.ToLower();
		}
	    }
	}

	protected override bool TermCompare(Term term) {
	    if ((object)field == (object)term.Field()) {
		string searchText = term.Text();
		if (!caseSensitive) {
		    searchText = searchText.ToLower();
		}

		return searchText.IndexOf(text) != -1;
	    }
	    endEnum = true;
	    return false;
	}
	
	public override float Difference() {
	    return 1.0f;
	}

	public override bool EndEnum() {
	    return endEnum;
	}
    }

    //<summary> Implements a substring query.  Searches for terms that
    //contains the specified term as a substring </summary>
    public class SubstringQuery : MultiTermQuery {
	public bool IsCaseSensitive = true;

	public SubstringQuery(Term term) : base (term) {
	}
	
	protected override FilteredTermEnum GetEnum(IndexReader reader) {
	    return new SubstringTermEnum(reader, GetTerm(), IsCaseSensitive);
	}	
    }
}
    
